==> stretch-1.00.tbz: stretch-1.00/stretch.c <==
// stretch.c
// makes new RGB image by  channel stretching
// heavily, and in a dirty(?) way, reuses code from resample.c
// sorry,
// Thu Jul 11 12:25:59 EDT 2002  <pac@next.gli.cas.cz>
// Version 1.00


#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>

#define ALPHA 3
#define RED 2
#define GREEN 1
#define BLUE 0

static int low, high;


void
usage(void)
{
	fprint(2, "usage: stretch from to  [imagefile]\n");
	fprint(2, "           jpg -3 testimg.jpg | stretch 0 512  | page  \n");
	exits("usage");
}

void
stretchchannel(uchar **in, int off, int iny, uchar **out, int outy, int n, int low, int high)
{
	int y, old, new;	

	switch(n){
	case 3:
	case 4: 
		for(y=0; y<outy; y++){
			old =  in[y][off];
			new = 255.0* old/(high-low) + 255-255*high/(high-low);
			if (new<0) new=0; 
			else if (new > 255) new = 255;
			out[y][off] = new;
		} 
	break;
	default:
		sysfatal("can't handle  %d-channel images, need 24(32)-bpp depth", n);	
	}
}

int
max(int a, int b)
{
	if(a > b)
		return a;
	return b;
}

Memimage*
mknewimg(int xsize, int ysize, Memimage *m)
{
	int i, j, bpl, nchan;
	Memimage *new;
	uchar **oscan, **nscan;

	new = allocmemimage(Rect(0, 0, xsize, ysize), m->chan);
	if(new == nil)
		sysfatal("can't allocate new image: %r");

	oscan = malloc(Dy(m->r)*sizeof(uchar*));
	nscan = malloc(max(ysize, Dy(m->r))*sizeof(uchar*));
	if(oscan == nil || nscan == nil)
		sysfatal("can't allocate: %r");

	/* unload original image into scan lines */
	bpl = bytesperline(m->r, m->depth);
	for(i=0; i<Dy(m->r); i++){
		oscan[i] = malloc(bpl);
		if(oscan[i] == nil)
			sysfatal("can't allocate: %r");
		j = unloadmemimage(m, Rect(m->r.min.x, m->r.min.y+i, m->r.max.x, m->r.min.y+i+1), oscan[i], bpl);
		if(j != bpl)
			sysfatal("unloadmemimage");
	}

	/* allocate scan lines for destination. we do y first, so need at least Dy(m->r) lines */
	bpl = bytesperline(Rect(0, 0, xsize, Dy(m->r)), m->depth);
	for(i=0; i<max(ysize, Dy(m->r)); i++){
		nscan[i] = malloc(bpl);
		if(nscan[i] == nil)
			sysfatal("can't allocate: %r");
	}

	nchan = m->depth/8;

	/* stretch channel in Y direction*/
	for(i=0; i<xsize; i++)
		for(j=0; j<nchan; j++)
			stretchchannel(oscan, nchan*i+j, Dy(m->r), nscan, ysize, nchan, low, high);

	/* pack data into destination */
	bpl = bytesperline(new->r, m->depth);
	for(i=0; i<ysize; i++){
		j = loadmemimage(new, Rect(0, i, xsize, i+1), nscan[i], bpl);
		if(j != bpl)
			sysfatal("loadmemimage: %r");
	}
	return new;
}

void
main(int argc, char *argv[])
{
	int i, fd, xsize, ysize, dummy;
	Rectangle rparam;
	Memimage *m, *new, *t1, *t2;
	char *file;
	ulong tchan;
	char tmp[100];
	double v;

	memimageinit();
	memset(&rparam, 0, sizeof rparam);
	xsize = ysize = 0;

	ARGBEGIN{
	}ARGEND

	file = "<stdin>";
	fd = 0;

	switch (argc) {
	case 3:
		file = argv[2];
		fd = open(file, OREAD);
		if(fd < 0)
			sysfatal("can't open %s: %r", file);
	case 2:
		low = atoi(argv[0]);
		high = atoi(argv[1]);
		break;
	default:
		usage();
	}

if (low > high) {
	dummy=high; high =  low; low = dummy;
}
	m = readmemimage(fd);
	if(m == nil)
		sysfatal("can't read %s: %r", file);

	xsize = Dx(m->r);
	ysize = Dy(m->r);

	new = nil;
	switch(m->chan){

	case RGB24:
	case RGBA32:
	case ARGB32:
	case XRGB32:
		new = mknewimg(xsize, ysize, m);
		break;

	case GREY1:
	case GREY2:
	case GREY4:
	case GREY8:
	case CMAP8:
	case RGB15:
	case RGB16:
		tchan = RGB24;
		goto Convert;

	Convert:
		/* use library to convert to byte-per-chan form, then convert back */
		t1 = allocmemimage(m->r, tchan);
		if(t1 == nil)
			sysfatal("can't allocate temporary image: %r");
		memimagedraw(t1, t1->r, m, m->r.min, nil, ZP);
		t2 = mknewimg(xsize, ysize, t1);
		freememimage(t1);
		new = allocmemimage(Rect(0, 0, xsize, ysize), m->chan);
		if(new == nil)
			sysfatal("can't allocate new image: %r");
		/* should do error diffusion here */
		memimagedraw(new, new->r, t2, t2->r.min, nil, ZP);
		freememimage(t2);
		break;

	default:
		sysfatal("can't handle channel type %s", chantostr(tmp, m->chan));
	}

	assert(new);
	if(writememimage(1, new) < 0)
		sysfatal("write error on output: %r");

	exits(nil);
}

==> stretch-1.01.tbz: stretch-1.01/stretch.c <==
// stretch.c
// makes new RGB image by  channel stretching
// heavily, and in a dirty(?) way, reuses code from resample.c
// sorry,
// Fri Jul 12 14:58:01 EDT 2002 <pac@next.gli.cas.cz>
// Version 1.01


#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>


static int low, high;


void
usage(void)
{
	fprint(2, "usage:\n\tstretch from to  [imagefile]\n");
	fprint(2, "\n\tjpg -3 testimg.jpg | stretch 0 512  | page  \n");
	fprint(2, "\n\tstretch -100 +128 rose.img  | page\n");
	exits("usage");
}

void
stretchchannel(uchar **in, int off, int iny, uchar **out, int outy, int n, int low, int high)
{
	int y, old, new;	

	switch(n){
	case 3:
	case 4: 
		for(y=0; y<outy; y++){
			old =  in[y][off];
			new = 255.0* old/(high-low) + 255-255*high/(high-low);
			if (new<0) new=0; 
			else if (new > 255) new = 255;
			out[y][off] = new;
		} 
	break;
	default:
		sysfatal("can't handle  %d-channel images, need 24(32)-bpp depth", n);	
	}
}

int
max(int a, int b)
{
	if(a > b)
		return a;
	return b;
}

Memimage*
mknewimg(int xsize, int ysize, Memimage *m)
{
	int i, j, bpl, nchan;
	Memimage *new;
	uchar **oscan, **nscan;

	new = allocmemimage(Rect(0, 0, xsize, ysize), m->chan);
	if(new == nil)
		sysfatal("can't allocate new image: %r");

	oscan = malloc(Dy(m->r)*sizeof(uchar*));
	nscan = malloc(max(ysize, Dy(m->r))*sizeof(uchar*));
	if(oscan == nil || nscan == nil)
		sysfatal("can't allocate: %r");

	/* unload original image into scan lines */
	bpl = bytesperline(m->r, m->depth);
	for(i=0; i<Dy(m->r); i++){
		oscan[i] = malloc(bpl);
		if(oscan[i] == nil)
			sysfatal("can't allocate: %r");
		j = unloadmemimage(m, Rect(m->r.min.x, m->r.min.y+i, m->r.max.x, m->r.min.y+i+1), oscan[i], bpl);
		if(j != bpl)
			sysfatal("unloadmemimage");
	}

	/* allocate scan lines for destination. we do y first, so need at least Dy(m->r) lines */
	bpl = bytesperline(Rect(0, 0, xsize, Dy(m->r)), m->depth);
	for(i=0; i<max(ysize, Dy(m->r)); i++){
		nscan[i] = malloc(bpl);
		if(nscan[i] == nil)
			sysfatal("can't allocate: %r");
	}

	nchan = m->depth/8;

	/* stretch channel in Y direction*/
	for(i=0; i<xsize; i++)
		for(j=0; j<nchan; j++)
			stretchchannel(oscan, nchan*i+j, Dy(m->r), nscan, ysize, nchan, low, high);

	/* pack data into destination */
	bpl = bytesperline(new->r, m->depth);
	for(i=0; i<ysize; i++){
		j = loadmemimage(new, Rect(0, i, xsize, i+1), nscan[i], bpl);
		if(j != bpl)
			sysfatal("loadmemimage: %r");
	}
	return new;
}

void
main(int argc, char *argv[])
{
	int i, fd, xsize, ysize, dummy;
	Rectangle rparam;
	Memimage *m, *new, *t1, *t2;
	char *file;
	ulong tchan;
	char tmp[100];
	double v;

	memimageinit();
	memset(&rparam, 0, sizeof rparam);
	xsize = ysize = 0;


	file = "<stdin>";
	fd = 0;

	switch (argc) {
	case 4:
		file = argv[3];
		fd = open(file, OREAD);
		if(fd < 0)
			sysfatal("can't open %s: %r", file);
	case 3:
		low = atoi(argv[1]);
		high = atoi(argv[2]);
		break;
	default:
		usage();
	}

if (low > high) {
	dummy=high; high =  low; low = dummy;
}
	m = readmemimage(fd);
	if(m == nil)
		sysfatal("can't read %s: %r", file);

	xsize = Dx(m->r);
	ysize = Dy(m->r);

	new = nil;
	switch(m->chan){

	case RGB24:
	case RGBA32:
	case ARGB32:
	case XRGB32:
		new = mknewimg(xsize, ysize, m);
		break;

	case GREY1:
	case GREY2:
	case GREY4:
	case GREY8:
	case CMAP8:
	case RGB15:
	case RGB16:
		tchan = RGB24;
		goto Convert;

	Convert:
		/* use library to convert to byte-per-chan form, then convert back */
		t1 = allocmemimage(m->r, tchan);
		if(t1 == nil)
			sysfatal("can't allocate temporary image: %r");
		memimagedraw(t1, t1->r, m, m->r.min, nil, ZP);
		t2 = mknewimg(xsize, ysize, t1);
		freememimage(t1);
		new = allocmemimage(Rect(0, 0, xsize, ysize), m->chan);
		if(new == nil)
			sysfatal("can't allocate new image: %r");
		/* should do error diffusion here */
		memimagedraw(new, new->r, t2, t2->r.min, nil, ZP);
		freememimage(t2);
		break;

	default:
		sysfatal("can't handle channel type %s", chantostr(tmp, m->chan));
	}

	assert(new);
	if(writememimage(1, new) < 0)
		sysfatal("write error on output: %r");

	exits(nil);
}
